ROOT=$(pwd)
mkdir $ROOT/build
BRIDGE_DIR=$ROOT/../
INSTALL_DIR=$ROOT/../../ios


export LIBRARY_OUTPUT_DIR=$ROOT/build
export KRAKEN_JS_ENGINE=jsc

release_build() {
  cmake -DCMAKE_BUILD_TYPE=RelWithDebInfo -DCMAKE_TOOLCHAIN_FILE=$BRIDGE_DIR/cmake/ios.toolchain.cmake -DPLATFORM=$KRAKEN_BRIDGE_ARCH -DENABLE_BITCODE=FALSE -G "Unix Makefiles" -B $ROOT/build/$1/cmake-build-release -S $BRIDGE_DIR
  cmake --build $ROOT/build/$1/cmake-build-release --config RelWithDebInfo --target kraken -- -j 4
}

build_simulator() {
  export KRAKEN_BRIDGE_ARCH=SIMULATOR64
  release_build x86_64
  mkdir $ROOT/build/x86_64/
  mv $ROOT/build/kraken_bridge.framework $ROOT/build/x86_64
}

build_armv7() {
  export KRAKEN_BRIDGE_ARCH=OS
  release_build armv7
  mkdir $ROOT/build/armv7/
  mv $ROOT/build/kraken_bridge.framework $ROOT/build/armv7
}

build_arm64() {
  export KRAKEN_BRIDGE_ARCH=OS64
  release_build arm64
  mkdir $ROOT/build/arm64/
  mv $ROOT/build/kraken_bridge.framework $ROOT/build/arm64
}

mergeDist() {
  SIMULATOR_BRIDGE=$ROOT/build/x86_64/kraken_bridge.framework/kraken_bridge
  ARMV7_BRIDGE=$ROOT/build/armv7/kraken_bridge.framework/kraken_bridge
  ARM64_BRIDGE=$ROOT/build/arm64/kraken_bridge.framework/kraken_bridge
  mkdir $ROOT/build/kraken_bridge.framework
  lipo -create $SIMULATOR_BRIDGE $ARMV7_BRIDGE $ARM64_BRIDGE -output $ROOT/build/kraken_bridge.framework/kraken_bridge
  cp $ROOT/support/kraken_bridge.plist $ROOT/build/kraken_bridge.framework
  dsymutil $ROOT/build/kraken_bridge.framework/kraken_bridge
  mv $ROOT/build/kraken_bridge.framework/kraken_bridge.dSYM $ROOT/build
  strip -S -X -x $ROOT/build/kraken_bridge.framework/kraken_bridge
}

install() {
  cp -r $ROOT/build/kraken_bridge.framework $INSTALL_DIR
  cp -r $ROOT/build/kraken_bridge.dSYM $INSTALL_DIR
}

build_simulator
build_armv7
build_arm64
mergeDist
install
